@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed-single.h,v 1.30 2005/02/14 20:57:07 chpe Exp $
+ *  $Id: ephy-embed-single.h,v 1.33 2005/10/12 19:45:06 chpe Exp $
  */
 
 #ifndef EPHY_EMBED_SINGLE_H
@@ -61,10 +61,6 @@ struct _EphyEmbedSingleIface
 				     char *mime_type,
 				     char *uri);
 
-	void	 (* network_status) (EphyEmbedSingle *single,
-				     gboolean offline);
-
-
         gboolean (* add_sidebar)    (EphyEmbedSingle *single,
 				     const char *url,
 				     const char *title);
@@ -88,12 +84,11 @@ struct _EphyEmbedSingleIface
 						 const char *features);
 	void		  (* clear_cache)	(EphyEmbedSingle *shell);
 	void		  (* clear_auth_cache)	(EphyEmbedSingle *shell);
-	void		  (* set_offline_mode)	(EphyEmbedSingle *shell,
+	void		  (* set_network_status)(EphyEmbedSingle *shell,
 						 gboolean offline);
-	gboolean	  (* get_offline_mode)	(EphyEmbedSingle *single);
+	gboolean	  (* get_network_status)(EphyEmbedSingle *single);
 	GList *		  (* get_font_list)	(EphyEmbedSingle *shell,
 						 const char *langGroup);
-	GList *		  (* get_printer_list)	(EphyEmbedSingle *shell);
 };
 
 GType		ephy_embed_single_get_type		(void);
@@ -108,16 +103,14 @@ void		ephy_embed_single_clear_cache		(EphyEmbedSingle *single);
 
 void		ephy_embed_single_clear_auth_cache	(EphyEmbedSingle *single);
 
-void		ephy_embed_single_set_offline_mode	(EphyEmbedSingle *single,
-							 gboolean offline);
+void		ephy_embed_single_set_network_status	(EphyEmbedSingle *single,
+							 gboolean online);
 
-gboolean	ephy_embed_single_get_offline_mode	(EphyEmbedSingle *single);
+gboolean	ephy_embed_single_get_network_status	(EphyEmbedSingle *single);
 
 GList		*ephy_embed_single_get_font_list	(EphyEmbedSingle *single,
 							 const char *lang_group);
 
-GList		*ephy_embed_single_get_printer_list	(EphyEmbedSingle *single);
-
 G_END_DECLS
 
 #endif
