@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.h,v 1.47 2005/08/27 19:23:42 chpe Exp $
+ *  $Id: EphyBrowser.h,v 1.49 2005/10/13 03:14:12 philipl Exp $
  */
 
 #ifndef EPHY_BROWSER_H
@@ -93,12 +93,12 @@ public:
 	EphyModalAlertEventListener(EphyBrowser *aOwner) : EphyEventListener(aOwner) { };
 };
 
-class EphyDOMContentLoadedEventListener : public EphyEventListener
+class EphyMiscDOMEventsListener : public EphyEventListener
 {
 public:
 	NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent);
 
-	EphyDOMContentLoadedEventListener(EphyBrowser *aOwner) : EphyEventListener(aOwner) { };
+	EphyMiscDOMEventsListener(EphyBrowser *aOwner) : EphyEventListener(aOwner) { };
 };
 
 class EphyDOMScrollEventListener : public EphyEventListener
@@ -129,7 +129,7 @@ class EphyBrowser
 {
 friend class EphyEventListener;
 friend class EphyDOMLinkEventListener;
-friend class EphyDOMContentLoadedEventListener;
+friend class EphyMiscDOMEventsListener;
 friend class EphyDOMScrollEventListener;
 friend class EphyPopupBlockEventListener;
 friend class EphyModalAlertEventListener;
@@ -177,6 +177,11 @@ public:
 	nsresult GetSecurityInfo (PRUint32 *aState, nsACString &aDescription);
 	nsresult ShowCertificate ();
 
+	nsresult CopySHistory (EphyBrowser *dest, PRBool copy_back,
+	                       PRBool copy_forward, PRBool copy_current);
+
+	nsresult Close ();
+
         EphyEmbedDocumentType GetDocumentType ();
 
 #ifndef HAVE_GECKO_1_8
@@ -193,7 +198,7 @@ private:
 	nsCOMPtr<nsIDOMEventTarget> mEventTarget;
 	nsCOMPtr<nsIDOMWindow> mDOMWindow;
 	EphyDOMLinkEventListener *mDOMLinkEventListener;
-	EphyDOMContentLoadedEventListener *mDOMContentLoadedEventListener;
+	EphyMiscDOMEventsListener *mMiscDOMEventsListener;
 	EphyDOMScrollEventListener *mDOMScrollEventListener;
 	EphyPopupBlockEventListener *mPopupBlockEventListener;
 	EphyModalAlertEventListener *mModalAlertListener;
