@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-string.h,v 1.12 2004/12/13 18:58:58 chpe Exp $
+ *  $Id: ephy-string.h,v 1.14 2005/10/11 13:12:36 chpe Exp $
  */
 
 #ifndef EPHY_STRING_H
@@ -25,10 +25,13 @@
 
 G_BEGIN_DECLS
 
-gboolean	 ephy_string_to_int			(const char *string,
-							 gulong *integer);
+gboolean  ephy_string_to_int	(const char *string,
+				 gulong *integer);
 
-char		*ephy_string_blank_chr			(char *source);
+char	 *ephy_string_blank_chr	(char *source);
+
+char	 *ephy_string_shorten	(char *str,
+				 gsize target_length);
 
 G_END_DECLS
 
