@@ -18,11 +18,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-location-entry.c,v 1.61 2005/07/26 14:41:39 chpe Exp $
+ *  $Id: ephy-location-entry.c,v 1.65 2005/10/06 18:28:55 chpe Exp $
  */
 
 #include "config.h"
 
+#include "ephy-icon-entry.h"
 #include "ephy-tree-model-node.h"
 #include "ephy-location-entry.h"
 #include "ephy-marshal.h"
@@ -48,7 +49,6 @@
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkseparatormenuitem.h>
-#include <gtk/gtkframe.h>
 #include <gtk/gtkalignment.h>
 #include <gtk/gtkclipboard.h>
 
@@ -59,13 +59,14 @@
 struct _EphyLocationEntryPrivate
 {
 	GtkTooltips *tips;
-	GtkWidget *ebox;
-	GtkWidget *entry;
+	EphyIconEntry *icon_entry;
 	GtkWidget *icon_ebox;
 	GtkWidget *icon;
 	GtkWidget *lock_ebox;
 	GtkWidget *lock;
 	GdkPixbuf *favicon;
+	GdkColor secure_bg_colour;
+	GdkColor secure_fg_colour;
 
 	char *before_completion;
 
@@ -75,8 +76,11 @@ struct _EphyLocationEntryPrivate
 	guint relevance_col;
 
 	guint hash;
+
 	guint user_changed : 1;
 	guint original_address : 1;
+	guint secure : 1;
+	guint apply_colours : 1;
 };
 
 static const struct
@@ -100,6 +104,8 @@ static const GtkTargetEntry url_drag_types [] =
 	{ EPHY_DND_TEXT_TYPE,       0, 2 }
 };
 
+static const GdkColor fallback_bg_colour = { 0, 0xf7f7, 0xf7f7, 0xbebe }; /* yellow-ish */
+
 static void ephy_location_entry_class_init (EphyLocationEntryClass *klass);
 static void ephy_location_entry_init (EphyLocationEntry *le);
 
@@ -156,11 +162,67 @@ ephy_location_entry_set_tooltip (GtkToolItem *tool_item,
 	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (tool_item);
 	EphyLocationEntryPrivate *priv = entry->priv;
 
-	gtk_tooltips_set_tip (tooltips, priv->entry, tip_text, tip_private);
+	gtk_tooltips_set_tip (tooltips, priv->icon_entry->entry,
+			      tip_text, tip_private);
 
 	return TRUE;
 }
 
+static void
+ephy_location_entry_style_set (GtkWidget *widget,
+			       GtkStyle *previous_style)
+{
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (widget);
+	EphyLocationEntryPrivate *priv = entry->priv;
+	GtkSettings *settings;
+	GdkColor *bg_colour = NULL, *fg_colour = NULL;
+	char *theme;
+	gboolean is_a11y_theme;
+
+	if (GTK_WIDGET_CLASS (parent_class)->style_set)
+	{
+		GTK_WIDGET_CLASS (parent_class)->style_set (widget, previous_style);
+	}
+
+	settings = gtk_settings_get_for_screen (gtk_widget_get_screen (widget));
+	g_object_get (settings, "gtk-theme-name", &theme, NULL);
+	is_a11y_theme = strncmp (theme, "HighContrast", strlen ("HighContrast")) == 0 ||
+			strncmp (theme, "LowContrast", strlen ("LowContrast")) == 0;
+	g_free (theme);
+
+	gtk_widget_style_get (widget,
+			      "secure-fg-color", &fg_colour,
+			      "secure-bg-color", &bg_colour,
+			      NULL);
+
+	/* We only use the fallback colours when we don't have an a11y theme */
+	priv->apply_colours = !is_a11y_theme || (fg_colour != NULL && bg_colour != NULL);
+
+	if (fg_colour != NULL)
+	{
+		priv->secure_fg_colour = *fg_colour;
+		gdk_color_free (fg_colour);
+	}
+	else
+	{
+		/* FIXME: fall back to black text maybe? */
+		priv->secure_fg_colour = widget->style->text[GTK_STATE_NORMAL];
+	}
+
+	if (bg_colour != NULL)
+	{
+		priv->secure_bg_colour = *bg_colour;
+		gdk_color_free (bg_colour);
+	}
+	else
+	{
+		priv->secure_bg_colour = fallback_bg_colour;
+	}
+
+	/* Apply the new style */
+	ephy_location_entry_set_secure (entry, priv->secure);
+}
+
 static void
 ephy_location_entry_finalize (GObject *object)
 {
@@ -181,12 +243,15 @@ static void
 ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 	GtkToolItemClass *tool_item_class = GTK_TOOL_ITEM_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = ephy_location_entry_finalize;
 
+	widget_class->style_set = ephy_location_entry_style_set;
+
 	tool_item_class->set_tooltip = ephy_location_entry_set_tooltip;
 
 	signals[USER_CHANGED] = g_signal_new (
@@ -229,6 +294,19 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 		0,
 		G_TYPE_NONE);
 
+        gtk_widget_class_install_style_property (widget_class,
+                                                 g_param_spec_boxed ("secure-bg-color",
+                                                                     "Secure background colour",
+                                                                     "Background colour to use for secure sites",
+                                                                     GDK_TYPE_COLOR,
+                                                                     G_PARAM_READABLE));
+
+        gtk_widget_class_install_style_property (widget_class,
+                                                 g_param_spec_boxed ("secure-fg-color",
+                                                                     "Secure foreground Colour",
+                                                                     "Foreground colour to use for secure sites",
+                                                                     GDK_TYPE_COLOR,
+                                                                     G_PARAM_READABLE));
 
 	g_type_class_add_private (object_class, sizeof (EphyLocationEntryPrivate));
 }
@@ -239,7 +317,7 @@ update_address_state (EphyLocationEntry *entry)
 	EphyLocationEntryPrivate *priv = entry->priv;
 	const char *text;
 
-	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
+	text = gtk_entry_get_text (GTK_ENTRY (priv->icon_entry->entry));
 	priv->original_address = text != NULL &&
 				 g_str_hash (text) == priv->hash;
 }
@@ -297,16 +375,20 @@ entry_key_press_cb (GtkEntry *entry,
 		    GdkEventKey *event,
 		    EphyLocationEntry *lentry)
 {
-	guint mask = gtk_accelerator_get_default_mod_mask ();
+	guint state = event->state & gtk_accelerator_get_default_mod_mask ();
 
-	if ((event->keyval == GDK_Return || event->keyval == GDK_ISO_Enter) &&
-	    (event->state & mask) == GDK_CONTROL_MASK)
+	if ((event->keyval == GDK_Return ||
+	     event->keyval == GDK_KP_Enter ||
+	     event->keyval == GDK_ISO_Enter) &&
+	    state == GDK_CONTROL_MASK)
 	{
+		gtk_im_context_reset (entry->im_context);
+
 		g_signal_emit_by_name (entry, "activate");
 
 		return TRUE;
 	}
-	else if (event->keyval == GDK_Escape && (event->state & mask) == 0)
+	else if (event->keyval == GDK_Escape && state == 0)
 	{
 		ephy_location_entry_reset (lentry);
 		/* don't return TRUE since we want to cancel the autocompletion popup too */
@@ -401,16 +483,17 @@ static gboolean
 match_selected_cb (GtkEntryCompletion *completion,
 		   GtkTreeModel *model,
 		   GtkTreeIter *iter,
-		   EphyLocationEntry *le)
+		   EphyLocationEntry *entry)
 {
+	EphyLocationEntryPrivate *priv = entry->priv;
 	char *item = NULL;
 
 	gtk_tree_model_get (model, iter,
-			    le->priv->action_col, &item, -1);
+			    priv->action_col, &item, -1);
 	if (item == NULL) return FALSE;
 
-	ephy_location_entry_set_location (le, item, NULL);
-	g_signal_emit_by_name (le->priv->entry, "activate");
+	ephy_location_entry_set_location (entry, item, NULL);
+	g_signal_emit_by_name (priv->icon_entry->entry, "activate");
 
 	g_free (item);
 
@@ -470,7 +553,7 @@ entry_clear_activate_cb (GtkMenuItem *item,
 	EphyLocationEntryPrivate *priv = entry->priv;
 
 	priv->user_changed = FALSE;
-	gtk_entry_set_text (GTK_ENTRY (priv->entry), "");
+	gtk_entry_set_text (GTK_ENTRY (priv->icon_entry->entry), "");
 	priv->user_changed = TRUE;
 }
 
@@ -484,6 +567,7 @@ entry_populate_popup_cb (GtkEntry *entry,
 	GtkWidget *menuitem;
 	GList *children, *item;
 	int pos = 0, sep = 0;
+	gboolean is_editable;
 
 	/* Clear and Copy mnemonics conflict, make custom menuitem */
 	image = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
@@ -497,8 +581,8 @@ entry_populate_popup_cb (GtkEntry *entry,
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(menuitem), image);
 	g_signal_connect (menuitem , "activate",
 			  G_CALLBACK (entry_clear_activate_cb), lentry);
-	gtk_widget_set_sensitive (menuitem,
-		       		  gtk_editable_get_editable (GTK_EDITABLE (priv->entry)));
+	is_editable = gtk_editable_get_editable (GTK_EDITABLE (priv->icon_entry->entry));
+	gtk_widget_set_sensitive (menuitem, is_editable);
 	gtk_widget_show (menuitem);
 
 	/* search for the 2nd separator (the one after Select All) in the context
@@ -562,41 +646,11 @@ lock_button_press_event_cb (GtkWidget *ebox,
 	return FALSE;
 }
 
-static void
-modify_background (EphyLocationEntry *entry)
-{
-	EphyLocationEntryPrivate *priv = entry->priv;
-
-	if (priv->entry->style == NULL || !GTK_WIDGET_REALIZED (priv->ebox)) return;
-
-	gtk_widget_modify_bg (priv->ebox, GTK_STATE_NORMAL,
-			      &priv->entry->style->base[GTK_STATE_NORMAL]);
-}
-
-static void
-entry_style_set_cb (GtkWidget *widget,
-		    GtkStyle *previous_style,
-		    EphyLocationEntry *entry)
-{
-	LOG ("entry_style_set_cb");
-
-	modify_background (entry);
-}
-
-static void
-entry_realize_cb (GtkWidget *widget,
-		  EphyLocationEntry *entry)
-{
-	LOG ("entry_realize_cb");
-
-	modify_background (entry);
-}
-
 static void
 ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
-	GtkWidget *alignment, *frame, *hbox;
+	GtkWidget *alignment;
 
 	LOG ("EphyLocationEntry constructing contents %p", entry);
 
@@ -604,22 +658,13 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 1, 1);
 	gtk_container_add (GTK_CONTAINER (entry), alignment);
 
-	frame = gtk_frame_new (NULL);
-	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
-	gtk_container_add (GTK_CONTAINER (alignment), frame);
-
-	priv->ebox = gtk_event_box_new ();
-	gtk_container_set_border_width (GTK_CONTAINER (priv->ebox), 0);
-	gtk_event_box_set_visible_window (GTK_EVENT_BOX (priv->ebox), TRUE);
-	gtk_container_add (GTK_CONTAINER (frame), priv->ebox);
-
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (priv->ebox), hbox);
+	priv->icon_entry = EPHY_ICON_ENTRY (ephy_icon_entry_new ());
+	gtk_container_add (GTK_CONTAINER (alignment),
+			   GTK_WIDGET (priv->icon_entry));
 
 	priv->icon_ebox = gtk_event_box_new ();
 	gtk_container_set_border_width (GTK_CONTAINER (priv->icon_ebox), 2);
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (priv->icon_ebox), FALSE);
-	gtk_box_pack_start (GTK_BOX (hbox), priv->icon_ebox, FALSE, FALSE, 2);
 	gtk_drag_source_set (priv->icon_ebox, GDK_BUTTON1_MASK,
 			     url_drag_types, G_N_ELEMENTS (url_drag_types),
 			     GDK_ACTION_ASK | GDK_ACTION_COPY | GDK_ACTION_LINK);
@@ -629,44 +674,38 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 	priv->icon = gtk_image_new ();
 	gtk_container_add (GTK_CONTAINER (priv->icon_ebox), priv->icon);
 
-	priv->entry = gtk_entry_new ();
-	gtk_entry_set_has_frame (GTK_ENTRY (priv->entry), FALSE);
-	gtk_box_pack_start (GTK_BOX (hbox), priv->entry, TRUE, TRUE, 0);
+	ephy_icon_entry_pack_widget (priv->icon_entry, priv->icon_ebox, TRUE);
 
 	priv->lock_ebox = gtk_event_box_new ();
 	gtk_container_set_border_width (GTK_CONTAINER (priv->lock_ebox), 2);
-        gtk_widget_add_events (priv->lock_ebox, GDK_BUTTON_PRESS_MASK);
+	gtk_widget_add_events (priv->lock_ebox, GDK_BUTTON_PRESS_MASK);
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (priv->lock_ebox), FALSE);
-	gtk_box_pack_end (GTK_BOX (hbox), priv->lock_ebox, FALSE, FALSE, 2);
 
 	priv->lock = gtk_image_new_from_stock (STOCK_LOCK_INSECURE,
 					       GTK_ICON_SIZE_MENU);
 	gtk_container_add (GTK_CONTAINER (priv->lock_ebox), priv->lock);
 
-	gtk_widget_show_all (alignment);
+	ephy_icon_entry_pack_widget (priv->icon_entry, priv->lock_ebox, FALSE);
 
-	/* monitor entry style so we can set the background behind the icons */
-	g_signal_connect_after (priv->entry, "style-set",
-				G_CALLBACK (entry_style_set_cb), entry);
-	g_signal_connect_after (priv->entry, "realize",
-				G_CALLBACK (entry_realize_cb), entry);
-
-	g_signal_connect (priv->icon_ebox, "drag_data_get",
+	g_signal_connect (priv->icon_ebox, "drag-data-get",
 			  G_CALLBACK (favicon_drag_data_get_cb), entry);
-	g_signal_connect (priv->entry, "populate_popup",
+	g_signal_connect (priv->lock_ebox, "button-press-event",
+			  G_CALLBACK (lock_button_press_event_cb), entry);
+
+	g_signal_connect (priv->icon_entry->entry, "populate_popup",
 			  G_CALLBACK (entry_populate_popup_cb), entry);
-	g_signal_connect (priv->entry, "key-press-event",
+	g_signal_connect (priv->icon_entry->entry, "key-press-event",
 			  G_CALLBACK (entry_key_press_cb), entry);
-	g_signal_connect (priv->entry, "button_press_event",
+	g_signal_connect (priv->icon_entry->entry, "button-press-event",
 			  G_CALLBACK (entry_button_press_cb), entry);
-	g_signal_connect (priv->entry, "changed",
+	g_signal_connect (priv->icon_entry->entry, "changed",
 			  G_CALLBACK (editable_changed_cb), entry);
-	g_signal_connect (priv->entry, "drag_motion",
+	g_signal_connect (priv->icon_entry->entry, "drag-motion",
 			  G_CALLBACK (entry_drag_motion_cb), entry);
-	g_signal_connect (priv->entry, "drag_drop",
+	g_signal_connect (priv->icon_entry->entry, "drag-drop",
 			  G_CALLBACK (entry_drag_drop_cb), entry);
-        g_signal_connect (priv->lock_ebox, "button-press-event",
-                          G_CALLBACK (lock_button_press_event_cb), entry);
+
+	gtk_widget_show_all (alignment);
 }
 
 static void
@@ -721,6 +760,7 @@ ephy_location_entry_set_completion (EphyLocationEntry *le,
 				    guint keywords_col,
 				    guint relevance_col)
 {
+	EphyLocationEntryPrivate *priv = le->priv;
 	GtkTreeModel *sort_model;
 	GtkEntryCompletion *completion;
 	GtkCellRenderer *cell;
@@ -749,7 +789,7 @@ ephy_location_entry_set_completion (EphyLocationEntry *le,
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
 				       cell, "text", text_col);
 
-	gtk_entry_set_completion (GTK_ENTRY (le->priv->entry), completion);
+	gtk_entry_set_completion (GTK_ENTRY (priv->icon_entry->entry), completion);
 	g_object_unref (completion);
 }
 
@@ -771,17 +811,19 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	 * bug #155824. So we save the selection iff the clipboard was owned by
 	 * the location entry.
 	 */
-	if (GTK_WIDGET_REALIZED (priv->entry))
+	if (GTK_WIDGET_REALIZED (GTK_WIDGET (priv->icon_entry)))
 	{
-		clipboard = gtk_widget_get_clipboard (priv->entry,
+		GtkWidget *gtkentry = priv->icon_entry->entry;
+
+		clipboard = gtk_widget_get_clipboard (gtkentry,
 						      GDK_SELECTION_PRIMARY);
 		g_return_if_fail (clipboard != NULL);
 
-		if (gtk_clipboard_get_owner (clipboard) == G_OBJECT (priv->entry) &&
-		    gtk_editable_get_selection_bounds (GTK_EDITABLE (priv->entry),
+		if (gtk_clipboard_get_owner (clipboard) == G_OBJECT (gtkentry) &&
+		    gtk_editable_get_selection_bounds (GTK_EDITABLE (gtkentry),
 	     					       &start, &end))
 		{
-			selection = gtk_editable_get_chars (GTK_EDITABLE (priv->entry),
+			selection = gtk_editable_get_chars (GTK_EDITABLE (gtkentry),
 							    start, end);
 		}
 	}
@@ -799,11 +841,13 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 		text = "";
 	}
 
+	/* First record the new hash, then update the entry text */
+	priv->hash = g_str_hash (address);
+
 	priv->user_changed = FALSE;
-	gtk_entry_set_text (GTK_ENTRY (priv->entry), text);
+	gtk_entry_set_text (GTK_ENTRY (priv->icon_entry->entry), text);
 	priv->user_changed = TRUE;
 
-	priv->hash = g_str_hash (address);
 	update_favicon (entry);
 
 	/* Now restore the selection.
@@ -818,9 +862,11 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 }
 
 const char *
-ephy_location_entry_get_location (EphyLocationEntry *le)
+ephy_location_entry_get_location (EphyLocationEntry *entry)
 {
-	return gtk_entry_get_text (GTK_ENTRY (le->priv->entry));
+	EphyLocationEntryPrivate *priv = entry->priv;
+
+	return gtk_entry_get_text (GTK_ENTRY (priv->icon_entry->entry));
 }
 
 gboolean
@@ -833,7 +879,7 @@ ephy_location_entry_reset (EphyLocationEntry *entry)
 
 	g_signal_emit (entry, signals[GET_LOCATION], 0, &url);
 	text = url != NULL ? url : "";
-	old_text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
+	old_text = gtk_entry_get_text (GTK_ENTRY (priv->icon_entry->entry));
 	old_text = old_text != NULL ? old_text : "";
 
 	retval = g_str_hash (text) != g_str_hash (old_text);
@@ -845,22 +891,25 @@ ephy_location_entry_reset (EphyLocationEntry *entry)
 }
 
 void
-ephy_location_entry_activate (EphyLocationEntry *le)
+ephy_location_entry_activate (EphyLocationEntry *entry)
 {
+	EphyLocationEntryPrivate *priv = entry->priv;
 	GtkWidget *toplevel;
 
-	toplevel = gtk_widget_get_toplevel (le->priv->entry);
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (entry));
 
-	gtk_editable_select_region (GTK_EDITABLE(le->priv->entry),
+	gtk_editable_select_region (GTK_EDITABLE(priv->icon_entry->entry),
 				    0, -1);
         gtk_window_set_focus (GTK_WINDOW(toplevel),
-                              le->priv->entry);
+                              priv->icon_entry->entry);
 }
 
 GtkWidget *
 ephy_location_entry_get_entry (EphyLocationEntry *entry)
 {
-	return entry->priv->entry;
+	EphyLocationEntryPrivate *priv = entry->priv;
+
+	return priv->icon_entry->entry;
 }
 
 void
@@ -879,11 +928,38 @@ ephy_location_entry_set_favicon (EphyLocationEntry *entry,
 	update_favicon (entry);
 }
 
+void
+ephy_location_entry_set_secure (EphyLocationEntry *entry,
+				gboolean secure)
+{
+	EphyLocationEntryPrivate *priv = entry->priv;
+	GtkWidget *widget = GTK_WIDGET (entry);
+	GtkWidget *gentry = ephy_icon_entry_get_entry (entry->priv->icon_entry);
+
+	priv->secure = secure;
+
+	/* We have to set the colour of the GtkEntry in the EphyIconEntry */
+	if (priv->secure && priv->apply_colours)
+	{
+		gtk_widget_modify_text (gentry, GTK_STATE_NORMAL, &priv->secure_fg_colour);
+		gtk_widget_modify_base (gentry, GTK_STATE_NORMAL, &priv->secure_bg_colour);
+	}
+	else
+	{
+		gtk_widget_modify_text (gentry, GTK_STATE_NORMAL, NULL);
+		gtk_widget_modify_base (gentry, GTK_STATE_NORMAL, NULL);
+	}
+
+	gtk_widget_queue_draw (widget);
+}
+
 void
 ephy_location_entry_set_show_lock (EphyLocationEntry *entry,
 				   gboolean show_lock)
 {
-	g_object_set (entry->priv->lock_ebox, "visible", show_lock, NULL);
+	EphyLocationEntryPrivate *priv = entry->priv;
+
+	g_object_set (priv->lock_ebox, "visible", show_lock, NULL);
 }
 
 void
