@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-view.c,v 1.62 2005/07/11 19:47:25 chpe Exp $
+ *  $Id: ephy-node-view.c,v 1.63 2005/10/01 07:38:05 chpe Exp $
  */
 
 #include "config.h"
@@ -570,7 +570,8 @@ ephy_node_view_key_press_cb (GtkTreeView *treeview,
 
 	if (event->keyval == GDK_space ||
 	    event->keyval == GDK_Return ||
-	    event->keyval == GDK_KP_Enter)
+	    event->keyval == GDK_KP_Enter ||
+	    event->keyval == GDK_ISO_Enter)
 	{
 		if (view->priv->toggle_column >= 0)
 		{
