@@ -2,6 +2,7 @@
  *  Copyright (C) 2003 Marco Pesenti Gritti
  *  Copyright (C) 2003, 2004 Christian Persch
  *  Copyright (C) 2004 Adam Hooper
+ *  Copyright (C) 2005 Crispin Flowerday
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-extensions-manager.c,v 1.25 2005/05/21 13:58:23 chpe Exp $
+ *  $Id: ephy-extensions-manager.c,v 1.37 2005/10/21 19:37:27 chpe Exp $
  */
 
 #include "config.h"
@@ -31,10 +32,17 @@
 #include "ephy-shell.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-file-helpers.h"
+#include "ephy-object-helpers.h"
 #include "ephy-debug.h"
 
 #include <libxml/tree.h>
 #include <libxml/xmlreader.h>
+#include <libxml/globals.h>
+#include <libxml/tree.h>
+#include <libxml/xmlwriter.h>
+#include <libxslt/xslt.h>
+#include <libxslt/transform.h>
+#include <libxslt/xsltutils.h>
 
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -45,20 +53,18 @@
 #include <dirent.h>
 #include <string.h>
 
-/* define to validate the extensions files */
-/* #define ENABLE_VALIDATION */
-
-#ifdef ENABLE_VALIDATION
-#include <libxml/xmlschemas.h>
-#endif
-
 #ifdef ENABLE_PYTHON
 #include "ephy-python-extension.h"
 #include "ephy-python-loader.h"
 #endif
 
 #define CONF_LOADED_EXTENSIONS	"/apps/epiphany/general/active_extensions"
-#define SCHEMA_FILE		"/epiphany-extension.xsd"
+#define EE_GROUP		"Epiphany Extension"
+#define DOT_INI			".ephy-extension"
+#define RELOAD_DELAY		333 /* ms */
+#define RELOAD_SYNC_DELAY	1000 /* ms */
+
+#define ENABLE_LEGACY_FORMAT
 
 #define EPHY_EXTENSIONS_MANAGER_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EXTENSIONS_MANAGER, EphyExtensionsManagerPrivate))
 
@@ -72,24 +78,20 @@ struct _EphyExtensionsManagerPrivate
 	GList *dir_monitors;
 	GList *windows;
 	guint active_extensions_notifier_id;
+	guint sync_timeout_id;
+	GHashTable *reload_hash;
 
-#ifdef ENABLE_VALIDATION
-	xmlSchemaPtr schema;
-	xmlSchemaValidCtxtPtr schema_ctxt;
+#ifdef ENABLE_LEGACY_FORMAT
+	xsltStylesheetPtr xml2ini_xsl;
 #endif
 };
 
 typedef struct
 {
 	EphyExtensionInfo info;
-	guint version;
-	gboolean load_deferred;
 	gboolean load_failed;
 
-	xmlChar *gettext_domain;
-	xmlChar *locale_directory;
-	xmlChar *loader_type;
-	GData *loader_attributes;
+	char *loader_type;
 
 	EphyLoader *loader; /* NULL if never loaded */
 	GObject *extension; /* NULL if unloaded */
@@ -267,15 +269,8 @@ free_extension_info (ExtensionInfo *info)
 	EphyExtensionInfo *einfo = (EphyExtensionInfo *) info;
 
 	g_free (einfo->identifier);
-	xmlFree ((xmlChar *) einfo->name);
-	xmlFree ((xmlChar *) einfo->description);
-	g_list_foreach (einfo->authors, (GFunc) xmlFree, NULL);
-	g_list_free (einfo->authors);
-	xmlFree ((xmlChar *) einfo->url);
-	xmlFree ((xmlChar *) info->gettext_domain);
-	xmlFree ((xmlChar *) info->locale_directory);
-	xmlFree ((xmlChar *) info->loader_type);
-	g_datalist_clear (&info->loader_attributes);
+	g_key_file_free (einfo->keyfile);
+	g_free (info->loader_type);
 
 	if (info->extension != NULL)
 	{
@@ -306,354 +301,52 @@ find_extension_info (const ExtensionInfo *info,
 	return strcmp (info->info.identifier, identifier);
 }
 
-typedef struct
-{
-	xmlChar *string;
-	guint match;
-} LocalisedString;
-
-static const char * const *languages = NULL;
-
 static void
-assign_localised_string (xmlTextReaderPtr reader,
-			 LocalisedString *string)
+ephy_extensions_manager_parse_keyfile (EphyExtensionsManager *manager,
+				       GKeyFile *key_file,
+				       const char *identifier)
 {
-	const xmlChar *value, *lang;
-	guint i;
-
-	lang = xmlTextReaderConstXmlLang (reader);
-	value = xmlTextReaderConstValue (reader);
-
-	if (G_UNLIKELY (lang == NULL))
-	{
-		/* languages always has "C" in it, so make sure we get a match */
-		lang = (const xmlChar *) "C";
-	}
-
-	for (i = 0; languages[i] != NULL && i < string->match; i++)
-	{
-		if (lang != NULL &&
-		    strcmp ((const char *) lang, languages[i]) == 0)
-		{
-			xmlFree (string->string);
-			string->string = xmlStrdup (value);
-			string->match = i;
-			break;
-		}
-	}
-}
-
-typedef enum
-{
-	STATE_START,
-	STATE_STOP,
-	STATE_ERROR,
-	STATE_EXTENSION,
-	STATE_NAME,
-	STATE_DESCRIPTION,
-	STATE_VERSION,
-	STATE_AUTHOR,
-	STATE_URL,
-	STATE_GETTEXT_DOMAIN,
-	STATE_LOCALE_DIRECTORY,
-	STATE_LOADER,
-	STATE_LOADER_ATTRIBUTE,
-	STATE_LOAD_DEFERRED,
-} ParserState;
-
-static void
-ephy_extensions_manager_load_string (EphyExtensionsManager *manager,
-				     const char *identifier,
-				     /* const */ char *xml)
-{
-	xmlDocPtr doc;
-	xmlTextReaderPtr reader;
-	ParserState state = STATE_START;
-	GQuark attr_quark = 0;
-	EphyExtensionInfo *einfo;
 	ExtensionInfo *info;
-	int ret;
-	LocalisedString description = { NULL, G_MAXUINT };
-	LocalisedString name = { NULL, G_MAXUINT };
+	EphyExtensionInfo *einfo;
+	char *start_group;
 
-	LOG ("Loading description file for '%s'", identifier);
+	LOG ("Parsing INI description file for '%s'", identifier);
 
-	if (g_list_find_custom (manager->priv->data, identifier,
-				(GCompareFunc) find_extension_info) != NULL)
+	start_group = g_key_file_get_start_group (key_file);
+	if (start_group == NULL ||
+	    strcmp (start_group, EE_GROUP) != 0 ||
+	    !g_key_file_has_group (key_file, "Loader"))
 	{
-		g_warning ("Extension description for '%s' already read!",
+		g_warning ("Invalid extension description file for '%s'; "
+			   "missing 'Epiphany Extension' or 'Loader' group",
 			   identifier);
+		
+		g_key_file_free (key_file);
+		g_free (start_group);
 		return;
 	}
+	g_free (start_group);
 
-	/* FIXME: Ideally we'd put the schema validator in the reader. libxml2
-	 * doesn't seem to support that at this point in time, so we've got to
-	 * put the schema validation on the Doc Tree and then pass that to the
-	 * reader. (maybe switch to RelaxNG?)
-	 */
-	doc = xmlParseDoc (xml);
-
-	if (doc == NULL)
+	if (!g_key_file_has_key (key_file, EE_GROUP, "Name", NULL) ||
+	    !g_key_file_has_key (key_file, EE_GROUP, "Description", NULL))
 	{
-		g_warning ("Couldn't read '%s' data\n", identifier);
+		g_warning ("Invalid extension description file for '%s'; "
+			   "missing 'Name' or 'Description' keys.",
+			   identifier);
+		
+		g_key_file_free (key_file);
 		return;
 	}
-#ifdef ENABLE_VALIDATION
-	/* Validate the extension description */
-	if (manager->priv->schema_ctxt)
-	{
-		if (xmlSchemaValidateDoc (manager->priv->schema_ctxt, doc))
-		{
-			g_warning ("Validation errors in '%s' data",
-				   identifier);
-			xmlFreeDoc (doc);
-			return;
-		}
-	}
-#endif
-	/* Now parse it */
-	reader = xmlReaderWalker (doc);
-	g_return_if_fail (reader != NULL);
 
 	info = g_new0 (ExtensionInfo, 1);
 	einfo = (EphyExtensionInfo *) info;
 	einfo->identifier = g_strdup (identifier);
-	g_datalist_init (&info->loader_attributes);
+	einfo->keyfile = key_file;
 
-	ret = xmlTextReaderRead (reader);
-
-	while (ret == 1)
-	{
-		const xmlChar *tag;
-		xmlReaderTypes type;
-
-		tag = xmlTextReaderConstName (reader);
-		type = xmlTextReaderNodeType (reader);
-
-		if (state == STATE_LOADER &&
-		    type == XML_READER_TYPE_ELEMENT &&
-		    xmlStrEqual (tag, (const xmlChar *) "attribute"))
-		{
-			xmlChar *name;
-
-			state = STATE_LOADER_ATTRIBUTE;
-
-			name = xmlTextReaderGetAttribute (reader, (const xmlChar *) "name");
-			attr_quark = g_quark_from_string ((const char *) name);
-			xmlFree (name);
-		}
-		else if (state == STATE_EXTENSION &&
-			 type == XML_READER_TYPE_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "author"))
-		{
-			state = STATE_AUTHOR;
-		}
-		else if (state == STATE_EXTENSION &&
-			 type == XML_READER_TYPE_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "description"))
-		{
-			state = STATE_DESCRIPTION;
-		}
-		else if (state == STATE_EXTENSION &&
-			 type == XML_READER_TYPE_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "gettext-domain"))
-		{
-			state = STATE_GETTEXT_DOMAIN;
-		}
-		else if (state == STATE_EXTENSION &&
-			 type == XML_READER_TYPE_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "load-deferred"))
-		{
-			state = STATE_LOAD_DEFERRED;
-		}
-		else if (state == STATE_EXTENSION &&
-			 type == XML_READER_TYPE_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "locale-directory"))
-		{
-			state = STATE_LOCALE_DIRECTORY;
-		}
-		else if (state == STATE_EXTENSION &&
-			 type == XML_READER_TYPE_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "name"))
-		{
-			state = STATE_NAME;
-		}
-		else if (state == STATE_EXTENSION &&
-			 type == XML_READER_TYPE_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "url"))
-		{
-			state = STATE_URL;
-		}
-		else if (state == STATE_EXTENSION &&
-			 type == XML_READER_TYPE_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "version"))
-		{
-			state = STATE_VERSION;
-		}
-		else if (state == STATE_EXTENSION &&
-			 type == XML_READER_TYPE_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "loader"))
-		{
-			state = STATE_LOADER;
-
-			info->loader_type = xmlTextReaderGetAttribute (reader, (const xmlChar *) "type");
-		}
-		else if (state == STATE_LOADER_ATTRIBUTE &&
-			 type == XML_READER_TYPE_TEXT &&
-			 attr_quark != 0)
-		{
-			xmlChar *value;
-
-			value = xmlTextReaderValue (reader);
-
-			g_datalist_id_set_data_full (&info->loader_attributes,
-						     attr_quark, value,
-						     (GDestroyNotify) xmlFree);
-			attr_quark = 0;
-		}
-		else if (state == STATE_LOADER_ATTRIBUTE &&
-			 type == XML_READER_TYPE_END_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "attribute"))
-		{
-			state = STATE_LOADER;
-		}
-		else if (state == STATE_AUTHOR &&
-			 type == XML_READER_TYPE_TEXT)
-		{
-			einfo->authors = g_list_prepend
-				(einfo->authors, xmlTextReaderValue (reader));
-		}
-		else if (state == STATE_DESCRIPTION &&
-			 type == XML_READER_TYPE_TEXT)
-		{
-			assign_localised_string (reader, &description);
-		}
-		else if (state == STATE_GETTEXT_DOMAIN &&
-			 type == XML_READER_TYPE_TEXT)
-		{
-			info->gettext_domain = xmlTextReaderValue (reader);
-		}
-		else if (state == STATE_LOAD_DEFERRED &&
-			 type == XML_READER_TYPE_TEXT)
-		{
-			const xmlChar *value;
-
-			value = xmlTextReaderConstValue (reader);
-			info->load_deferred =
-				(value != NULL && xmlStrEqual (value, (const xmlChar *) "true"));
-		}
-		else if (state == STATE_LOCALE_DIRECTORY &&
-			 type == XML_READER_TYPE_TEXT)
-		{
-			info->locale_directory = xmlTextReaderValue (reader);
-		}
-		else if (state == STATE_NAME &&
-			 type == XML_READER_TYPE_TEXT)
-		{
-			assign_localised_string (reader, &name);
-		}
-		else if (state == STATE_VERSION &&
-			 type == XML_READER_TYPE_TEXT)
-		{
-			info->version = (guint) strtol ((const char *) xmlTextReaderConstValue (reader), NULL, 10);
-		}
-		else if (state == STATE_URL &&
-			 type == XML_READER_TYPE_TEXT)
-		{
-			einfo->url = xmlTextReaderValue (reader);
-		}
-		else if (state == STATE_AUTHOR &&
-			 type == XML_READER_TYPE_END_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "author"))
-		{
-			state = STATE_EXTENSION;
-		}
-		else if (state == STATE_DESCRIPTION &&
-			 type == XML_READER_TYPE_END_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "description"))
-		{
-			state = STATE_EXTENSION;
-		}
-		else if (state == STATE_GETTEXT_DOMAIN &&
-			 type == XML_READER_TYPE_END_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "gettext-domain"))
-		{
-			state = STATE_EXTENSION;
-		}
-		else if (state == STATE_LOCALE_DIRECTORY &&
-			 type == XML_READER_TYPE_END_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "locale-directory"))
-		{
-			state = STATE_EXTENSION;
-		}
-		else if (state == STATE_LOADER &&
-			 type == XML_READER_TYPE_END_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "loader"))
-		{
-			state = STATE_EXTENSION;
-		}
-		else if (state == STATE_NAME &&
-			 type == XML_READER_TYPE_END_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "name"))
-		{
-			state = STATE_EXTENSION;
-		}
-		else if (state == STATE_URL &&
-			 type == XML_READER_TYPE_END_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "url"))
-		{
-			state = STATE_EXTENSION;
-		}
-		else if (state == STATE_VERSION &&
-			 type == XML_READER_TYPE_END_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "version"))
-		{
-			state = STATE_EXTENSION;
-		}
-		else if (type == XML_READER_TYPE_SIGNIFICANT_WHITESPACE ||
-			 type == XML_READER_TYPE_WHITESPACE ||
-			 type == XML_READER_TYPE_TEXT)
-		{
-			/* eat it */
-		}
-		else if (state == STATE_START &&
-			 type == XML_READER_TYPE_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "extension"))
-		{
-			state = STATE_EXTENSION;
-		}
-		else if (state == STATE_EXTENSION &&
-			 type == XML_READER_TYPE_END_ELEMENT &&
-			 xmlStrEqual (tag, (const xmlChar *) "extension"))
-		{
-			state = STATE_STOP;
-		}
-		else
-		{
-			const xmlChar *content;
-
-			content = xmlTextReaderConstValue (reader);
-			g_warning ("tag '%s' of type %d in state %d with content '%s' was unexpected!",
-				   tag, type, state, content ? (char *) content : "(null)");
-
-			state = STATE_ERROR;
-			break;
-		}
-
-		ret = xmlTextReaderRead (reader);
-	}
-
-	xmlFreeTextReader (reader);
-	xmlFreeDoc (doc);
-
-	/* assign localised strings */
-	einfo->description = description.string;
-	einfo->name = name.string;
+	info->loader_type = g_key_file_get_string (key_file, "Loader", "Type", NULL);
 
 	/* sanity check */
-	if (ret < 0 || state != STATE_STOP ||
-	    einfo->name == NULL || einfo->description == NULL ||
-	    info->loader_type == NULL || info->loader_type[0] == '\0')
+	if (info->loader_type == NULL || info->loader_type[0] == '\0')
 	{
 		free_extension_info (info);
 		return;
@@ -664,13 +357,108 @@ ephy_extensions_manager_load_string (EphyExtensionsManager *manager,
 	g_signal_emit (manager, signals[ADDED], 0, info);
 }
 
+static void
+ephy_extensions_manager_load_ini_file (EphyExtensionsManager *manager,
+				       const char *identifier,
+				       const char *path)
+{
+	GKeyFile *keyfile;
+	GError *err = NULL;
+
+	keyfile = g_key_file_new ();
+	if (!g_key_file_load_from_file (keyfile, path, G_KEY_FILE_NONE, &err))
+	{
+		g_warning ("Could load key file for '%s': '%s'",
+			   identifier, err->message);
+		g_error_free (err);
+		g_key_file_free (keyfile);
+		return;
+	}
+
+	ephy_extensions_manager_parse_keyfile (manager, keyfile, identifier);
+}
+
+#ifdef ENABLE_LEGACY_FORMAT
+
+static void
+ephy_extensions_manager_load_xml_file (EphyExtensionsManager *manager,
+				       const char *identifier,
+				       const char *path)
+{
+	EphyExtensionsManagerPrivate *priv = manager->priv;
+	xmlDocPtr doc, res;
+	const xmlChar *xsl_file;
+	xmlChar *output = NULL;
+	int outlen = -1, ret = -1;
+	
+	START_PROFILER ("Transforming .xml -> " DOT_INI)
+
+	doc = xmlParseFile (path);
+	if (!doc) goto out;
+
+	if (priv->xml2ini_xsl == NULL)
+	{
+		xsl_file = (const xmlChar *) ephy_file ("ephy-xml2ini.xsl");
+		if (!xsl_file) return;
+
+		priv->xml2ini_xsl = xsltParseStylesheetFile (xsl_file);
+		if (priv->xml2ini_xsl == NULL)
+		{
+			g_warning ("Couldn't parse the XSL to transform .xml extension descriptions!\n");
+			goto out;
+		}
+	}
+
+	res = xsltApplyStylesheet (priv->xml2ini_xsl, doc, NULL);
+	if (!res) goto out;
+
+	ret = xsltSaveResultToString (&output, &outlen, res, priv->xml2ini_xsl);
+
+	if (ret >= 0 && output != NULL && outlen > -1)
+	{
+		GKeyFile *keyfile;
+		GError *err = NULL;
+
+		keyfile = g_key_file_new ();
+		if (!g_key_file_load_from_data (keyfile, (char *) output, outlen,
+					        G_KEY_FILE_NONE, &err))
+		{
+			g_warning ("Could load converted key file for '%s': '%s'",
+				   identifier, err->message);
+			g_error_free (err);
+			g_key_file_free (keyfile);
+			goto out;
+		}
+
+		ephy_extensions_manager_parse_keyfile (manager, keyfile, identifier);
+	}
+
+	xmlFreeDoc (res);
+	xmlFreeDoc (doc);
+
+out:
+	xmlFree (output);
+
+	STOP_PROFILER ("Transforming .xml -> " DOT_INI)
+}
+
+#endif /* ENABLE_LEGACY_FORMAT */
+
 static char *
 path_to_identifier (const char *path)
 {
 	char *identifier, *dot;
 
 	identifier = g_path_get_basename (path);
-	dot = strstr (identifier, ".xml");
+	dot = strstr (identifier, DOT_INI);
+
+#ifdef ENABLE_LEGACY_FORMAT
+	if (!dot)
+	{
+		dot = strstr (identifier, ".xml");
+	}
+#endif
+
 	g_return_val_if_fail (dot != NULL, NULL);
 
 	*dot = '\0';
@@ -683,28 +471,36 @@ ephy_extensions_manager_load_file (EphyExtensionsManager *manager,
 				   const char *path)
 {
 	char *identifier;
-	char *xml;
-	GError *err = NULL;
-
-	g_file_get_contents (path, &xml, NULL, &err);
-
-	if (err != NULL)
-	{
-		g_warning ("Could not read file at '%s': '%s'",
-			   path, err->message);
-		g_error_free (err);
-		return;
-	}
 
 	identifier = path_to_identifier (path);
 	g_return_if_fail (identifier != NULL);
+	if (identifier == NULL) return;
 
-	ephy_extensions_manager_load_string (manager, identifier, xml);
+	if (g_list_find_custom (manager->priv->data, identifier,
+	    (GCompareFunc) find_extension_info) != NULL)
+	{
+		g_warning ("Extension description for '%s' already read!",
+			   identifier);
+		return;
+	}
+
+	if (g_str_has_suffix (path, DOT_INI))
+	{
+		ephy_extensions_manager_load_ini_file (manager, identifier,
+						       path);
+	}
+#ifdef ENABLE_LEGACY_FORMAT
+	else if (g_str_has_suffix (path, ".xml"))
+	{
+		ephy_extensions_manager_load_xml_file (manager, identifier,
+						       path);
+	}
+#endif
 
 	g_free (identifier);
-	g_free (xml);
 }
 
+
 static int
 find_loader (const LoaderInfo *info,
 	      const char *type)
@@ -732,8 +528,8 @@ get_loader_for_type (EphyExtensionsManager *manager,
 {
 	LoaderInfo *info;
 	GList *l;
-	GData *attr = NULL;
-	char *path, *name, *stype;
+	char *path, *name, *stype, *data;
+	GKeyFile *keyfile;
 	EphyLoader *shlib_loader;
 	GObject *loader;
 
@@ -774,21 +570,27 @@ get_loader_for_type (EphyExtensionsManager *manager,
 #endif
 	}
 
-	stype = sanitise_type (type);
-	name = g_strconcat ("lib", stype, "loader.", G_MODULE_SUFFIX, NULL);
-	path = g_build_filename (LOADER_DIR, name, NULL);
-	g_datalist_init (&attr);
-	g_datalist_set_data (&attr, "library", path);
-
 	shlib_loader = get_loader_for_type (manager, "shlib");
 	g_return_val_if_fail (shlib_loader != NULL, NULL);
 
-	loader = ephy_loader_get_object (shlib_loader, &attr);
-	g_datalist_clear (&attr);
+	stype = sanitise_type (type);
+	name = g_strconcat ("lib", stype, "loader.", G_MODULE_SUFFIX, NULL);
+	path = g_build_filename (LOADER_DIR, name, NULL);
+	data = g_strconcat ("[Loader]\nType=shlib\nLibrary=", path, "\n", NULL);
 	g_free (stype);
 	g_free (name);
 	g_free (path);
 
+	keyfile = g_key_file_new ();
+	if (!g_key_file_load_from_data (keyfile, data, strlen (data), 0, NULL))
+	{
+		g_free (data);
+		return NULL;
+	}
+
+	loader = ephy_loader_get_object (shlib_loader, keyfile);
+	g_key_file_free (keyfile);
+
 	if (EPHY_IS_LOADER (loader))
 	{
 		info = g_new (LoaderInfo, 1);
@@ -847,8 +649,7 @@ load_extension (EphyExtensionsManager *manager,
 
 	info->loader = loader;
 
-	info->extension = ephy_loader_get_object (loader,
-						   &info->loader_attributes);
+	info->extension = ephy_loader_get_object (loader, info->info.keyfile);
 
 	/* attach if the extension implements EphyExtensionIface */
 	if (EPHY_IS_EXTENSION (info->extension))
@@ -891,14 +692,6 @@ detach_window (EphyWindow *window,
 	ephy_extension_detach_window (extension, window);
 }
 
-static gboolean
-idle_unref (GObject *object)
-{
-	g_object_unref (object);
-
-	return FALSE;
-}
-
 static void
 unload_extension (EphyExtensionsManager *manager,
 		  ExtensionInfo *info)
@@ -925,7 +718,7 @@ unload_extension (EphyExtensionsManager *manager,
 		 * extension has its own functions queued in the idle loop, the
 		 * functions must exist in memory before being called.
 		 */
-		g_idle_add ((GSourceFunc) idle_unref, info->extension);
+		ephy_object_idle_unref (info->extension);
 	}
 
 	ephy_loader_release_object (info->loader, G_OBJECT (info->extension));
@@ -1035,57 +828,96 @@ ephy_extensions_manager_unload_file (EphyExtensionsManager *manager,
 	g_free (identifier);
 }
 
-static void
-load_file_from_monitor (EphyExtensionsManager *manager,
-			const char *path)
+static gboolean
+reload_sync_cb (EphyExtensionsManager *manager)
 {
+	EphyExtensionsManagerPrivate *priv = manager->priv;
+
+	if (priv->sync_timeout_id != 0)
+	{
+		g_source_remove (priv->sync_timeout_id);
+		priv->sync_timeout_id = 0;
+	}
+
+	sync_loaded_extensions (manager);
+
+	return FALSE;
+}
+
+static gboolean
+reload_cb (gpointer *data)
+{
+	EphyExtensionsManager *manager = EPHY_EXTENSIONS_MANAGER (data[0]);
+	EphyExtensionsManagerPrivate *priv = manager->priv;
+	char *path = data[1];
+
+	LOG ("Reloading %s", path);
+
+	/* We still need path and don't want to remove the timeout
+	 * which will be removed automatically when we return, so 
+	 * just use _steal instead of _remove.
+	 */
+	g_hash_table_steal (priv->reload_hash, path);
+
+	ephy_extensions_manager_load_file (manager, path);
+	g_free (path);
+
+	/* Schedule a sync of active extensions */
+	/* FIXME: just look if we need to activate *this* extension? */
+
+	if (priv->sync_timeout_id != 0)
+	{
+		g_source_remove (priv->sync_timeout_id);
+	}
+
+	priv->sync_timeout_id = g_timeout_add (RELOAD_SYNC_DELAY,
+					       (GSourceFunc) reload_sync_cb,
+					       manager);
+	return FALSE;
+}
+
+static void
+schedule_load_from_monitor (EphyExtensionsManager *manager,
+			    const char *path)
+{
+	EphyExtensionsManagerPrivate *priv = manager->priv;
+	char *identifier, *copy;
+	gpointer *data;
+	guint timeout_id;
+
 	/* When a file is installed, it sometimes gets CREATED empty and then
 	 * gets its contents filled later (for a CHANGED signal). Theoretically
 	 * I suppose we could get a CHANGED signal when the file is half-full,
-	 * but I doubt that'll happen much (the xml files are <1000 bytes). We
-	 * don't want warnings all over the place, so we return from this
-	 * function if the file is empty. (We're assuming that if a file is
+	 * but I doubt that'll happen much (the files are <1000 bytes). We
+	 * don't want warnings all over the place, so we just wait a bit before
+	 * actually reloading the file. (We're assuming that if a file is
 	 * empty it'll be filled soon and this function will be called again.)
 	 *
 	 * Oh, and we return if the extension is already loaded, too.
 	 */
-	char *identifier;
-	char *xml;
-	gsize len;
-	GError *err = NULL;
-
-	g_file_get_contents (path, &xml, &len, &err);
-
-	if (err != NULL)
-	{
-		g_warning  ("Could not read file at '%s': '%s'",
-			    path, err->message);
-		g_error_free (err);
-		return;
-	}
-
-	if (len == 0) {
-		g_free (xml);
-		return;
-	}
 
 	identifier = path_to_identifier (path);
 	g_return_if_fail (identifier != NULL);
+	if (identifier == NULL) return;
 
 	if (g_list_find_custom (manager->priv->data, identifier,
 				(GCompareFunc) find_extension_info) != NULL)
 	{
 		g_free (identifier);
-		g_free (xml);
 		return;
 	}
-
-	ephy_extensions_manager_load_string (manager, identifier, xml);
-
 	g_free (identifier);
-	g_free (xml);
 
-	sync_loaded_extensions (manager);
+	g_return_if_fail (priv->reload_hash != NULL);
+
+	data = g_new (gpointer, 2);
+	data[0] = (gpointer) manager;
+	data[1] = copy = g_strdup (path);
+	timeout_id = g_timeout_add_full (G_PRIORITY_LOW, RELOAD_DELAY,
+					 (GSourceFunc) reload_cb,
+					 data, (GDestroyNotify) g_free);
+	g_hash_table_replace (priv->reload_hash, copy /* owns it */,
+			      GUINT_TO_POINTER (timeout_id));
 }
 
 static void
@@ -1098,18 +930,20 @@ dir_changed_cb (GnomeVFSMonitorHandle *handle,
 	char *path;
 
 	/*
-	 * We only deal with XML files: Add them to the manager when created,
-	 * remove them when deleted.
+	 * We only deal with XML and INI files:
+	 * Add them to the manager when created, remove them when deleted.
 	 */
-	if (g_str_has_suffix (info_uri, ".xml") == FALSE) return;
+	if (g_str_has_suffix (info_uri, DOT_INI) == FALSE &&
+	    g_str_has_suffix (info_uri, ".xml") == FALSE) return;
 
 	path = gnome_vfs_get_local_path_from_uri (info_uri);
+	g_print ("notify, uri=%s path=%s\n", info_uri, path);
 
 	switch (event_type)
 	{
 		case GNOME_VFS_MONITOR_EVENT_CREATED:
 		case GNOME_VFS_MONITOR_EVENT_CHANGED:
-			load_file_from_monitor (manager, path);
+			schedule_load_from_monitor (manager, path);
 			break;
 		case GNOME_VFS_MONITOR_EVENT_DELETED:
 			ephy_extensions_manager_unload_file (manager, path);
@@ -1143,7 +977,8 @@ ephy_extensions_manager_load_dir (EphyExtensionsManager *manager,
 	}
 	while ((e = readdir (d)) != NULL)
 	{
-		if (g_str_has_suffix (e->d_name, ".xml"))
+		if (g_str_has_suffix (e->d_name, DOT_INI) ||
+		    g_str_has_suffix (e->d_name, ".xml"))
 		{
 			file_path = g_build_filename (path, e->d_name, NULL);
 			ephy_extensions_manager_load_file (manager, file_path);
@@ -1178,71 +1013,24 @@ active_extensions_notifier (GConfClient *client,
 	sync_loaded_extensions (manager);
 }
 
-#ifdef ENABLE_VALIDATION
 static void
-xml_error_cb (EphyExtensionsManager *manager,
-	      const char *msg,
-	      ...)
-
+cancel_timeout (gpointer data)
 {
-	va_list args;
+	guint id = GPOINTER_TO_UINT (data);
 
-	va_start (args, msg);
-	g_logv (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING, msg, args);
-	va_end(args);
+	g_source_remove (id);
 }
 
-static void
-init_schema_ctxt (EphyExtensionsManager *manager)
-{
-	xmlSchemaParserCtxtPtr parse_ctxt;
-	const char *filename;
-
-	manager->priv->schema = NULL;
-	manager->priv->schema_ctxt = NULL;
-
-	filename = ephy_file (SCHEMA_FILE);
-	g_return_if_fail (filename != NULL);
-
-	parse_ctxt = xmlSchemaNewParserCtxt (filename);
-	if (parse_ctxt == NULL)
-	{
-		g_warning ("Error opening extensions description schema file "
-			   "\"" SCHEMA_FILE "\"");
-		return;
-	}
-
-	manager->priv->schema = xmlSchemaParse (parse_ctxt);
-	xmlSchemaFreeParserCtxt (parse_ctxt);
-	if (manager->priv->schema == NULL)
-	{
-		g_warning ("Error parsing extensions description schema file "
-			   "\"" SCHEMA_FILE "\"");
-		return;
-	}
-
-	manager->priv->schema_ctxt = xmlSchemaNewValidCtxt
-		(manager->priv->schema);
-	if (manager->priv->schema == NULL)
-	{
-		g_warning ("Error creating extensions description schema "
-			   "validation context for \"" SCHEMA_FILE "\"");
-		return;
-	}
-
-	xmlSchemaSetValidErrors (manager->priv->schema_ctxt,
-				 (xmlSchemaValidityErrorFunc) xml_error_cb,
-				 (xmlSchemaValidityWarningFunc) xml_error_cb,
-				 manager);
-}
-#endif
-
 static void
 ephy_extensions_manager_init (EphyExtensionsManager *manager)
 {
-	manager->priv = EPHY_EXTENSIONS_MANAGER_GET_PRIVATE (manager);
+	EphyExtensionsManagerPrivate *priv;
 
-	LOG ("EphyExtensionsManager initialising");
+	priv = manager->priv = EPHY_EXTENSIONS_MANAGER_GET_PRIVATE (manager);
+
+	priv->reload_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
+						   (GDestroyNotify) g_free,
+						   (GDestroyNotify) cancel_timeout);
 }
 
 void
@@ -1254,10 +1042,6 @@ ephy_extensions_manager_startup (EphyExtensionsManager *manager)
 
 	LOG ("EphyExtensionsManager startup");
 
-#ifdef ENABLE_VALIDATION
-	init_schema_ctxt (manager);
-#endif
-
 	/* load the extensions descriptions */
 	path = g_build_filename (ephy_dot_dir (), "extensions", NULL);
 	ephy_extensions_manager_load_dir (manager, path);
@@ -1274,42 +1058,73 @@ ephy_extensions_manager_startup (EphyExtensionsManager *manager)
 }
 
 static void
-ephy_extensions_manager_finalize (GObject *object)
+ephy_extensions_manager_dispose (GObject *object)
 {
 	EphyExtensionsManager *manager = EPHY_EXTENSIONS_MANAGER (object);
 	EphyExtensionsManagerPrivate *priv = manager->priv;
 
-	LOG ("EphyExtensionsManager finalising");
-
-	eel_gconf_notification_remove (manager->priv->active_extensions_notifier_id);
-
-	g_list_foreach (priv->dir_monitors, (GFunc) gnome_vfs_monitor_cancel, NULL);
-	g_list_free (priv->dir_monitors);
-
-	g_list_foreach (priv->extensions, (GFunc) g_object_unref, NULL);
-	g_list_free (priv->extensions);
-
-	/* FIXME release loaded loaders */
-	g_list_foreach (priv->factories, (GFunc) free_loader_info, NULL);
-	g_list_free (priv->factories);
-
-	g_list_foreach (priv->data, (GFunc) free_extension_info, NULL);
-	g_list_free (priv->data);
-
-	g_list_free (priv->windows);
-
-#ifdef ENABLE_VALIDATION
-	if (priv->schema)
+#ifdef ENABLE_LEGACY_FORMAT
+	if (priv->xml2ini_xsl != NULL)
 	{
-		xmlSchemaFree (priv->schema);
-	}
-	if (priv->schema_ctxt)
-	{
-		xmlSchemaFreeValidCtxt (priv->schema_ctxt);
+		xsltFreeStylesheet (priv->xml2ini_xsl);
+		priv->xml2ini_xsl = NULL;
 	}
 #endif
 
-	parent_class->finalize (object);
+	if (priv->active_extensions_notifier_id != 0)
+	{
+		eel_gconf_notification_remove (priv->active_extensions_notifier_id);
+		priv->active_extensions_notifier_id = 0;
+	}
+
+	if (priv->reload_hash != NULL)
+	{
+		g_hash_table_destroy (priv->reload_hash);
+		priv->reload_hash = NULL;
+	}
+
+	if (priv->sync_timeout_id != 0)
+	{
+		g_source_remove (priv->sync_timeout_id);
+		priv->sync_timeout_id = 0;
+	}
+
+	if (priv->dir_monitors != NULL)
+	{
+		g_list_foreach (priv->dir_monitors, (GFunc) gnome_vfs_monitor_cancel, NULL);
+		g_list_free (priv->dir_monitors);
+		priv->dir_monitors = NULL;
+	}
+
+	if (priv->extensions != NULL)
+	{
+		g_list_foreach (priv->extensions, (GFunc) g_object_unref, NULL);
+		g_list_free (priv->extensions);
+		priv->extensions = NULL;
+	}
+
+	if (priv->factories != NULL)
+	{
+		/* FIXME release loaded loaders */
+		g_list_foreach (priv->factories, (GFunc) free_loader_info, NULL);
+		g_list_free (priv->factories);
+		priv->factories = NULL;
+	}
+
+	if (priv->data != NULL)
+	{
+		g_list_foreach (priv->data, (GFunc) free_extension_info, NULL);
+		g_list_free (priv->data);
+		priv->data = NULL;
+	}
+
+	if (priv->windows != NULL)
+	{
+		g_list_free (priv->windows);
+		priv->windows = NULL;
+	}
+
+	parent_class->dispose (object);
 }
 
 static void
@@ -1325,7 +1140,7 @@ impl_attach_window (EphyExtension *extension,
 {
 	EphyExtensionsManager *manager = EPHY_EXTENSIONS_MANAGER (extension);
 
-	LOG ("Attach");
+	LOG ("Attach window %p", window);
 
 	g_list_foreach (manager->priv->extensions,
 			(GFunc) attach_extension_to_window, window);
@@ -1340,7 +1155,7 @@ impl_detach_window (EphyExtension *extension,
 	EphyExtensionsManager *manager = EPHY_EXTENSIONS_MANAGER (extension);
 	GList *tabs, *l;
 
-	LOG ("Detach");
+	LOG ("Detach window %p", window);
 
 	manager->priv->windows = g_list_remove (manager->priv->windows, window);
 
@@ -1370,7 +1185,7 @@ impl_attach_tab (EphyExtension *extension,
 	EphyExtensionsManager *manager = EPHY_EXTENSIONS_MANAGER (extension);
 	GList *l;
 
-	LOG ("Attach tab");
+	LOG ("Attach window %p tab %p", window, tab);
 
 	for (l = manager->priv->extensions; l; l = l->next)
 	{
@@ -1387,7 +1202,7 @@ impl_detach_tab (EphyExtension *extension,
 	EphyExtensionsManager *manager = EPHY_EXTENSIONS_MANAGER (extension);
 	GList *l;
 
-	LOG ("Detach Tab");
+	LOG ("Detach window %p tab %p", window, tab);
 
 	g_object_ref (window);
 	g_object_ref (tab);
@@ -1418,7 +1233,7 @@ ephy_extensions_manager_class_init (EphyExtensionsManagerClass *class)
 
 	parent_class = (GObjectClass *) g_type_class_peek_parent (class);
 
-	object_class->finalize = ephy_extensions_manager_finalize;
+	object_class->dispose = ephy_extensions_manager_dispose;
 
 	signals[CHANGED] =
 		g_signal_new ("changed",
@@ -1452,7 +1267,4 @@ ephy_extensions_manager_class_init (EphyExtensionsManagerClass *class)
 			      G_TYPE_POINTER);
 	
 	g_type_class_add_private (object_class, sizeof (EphyExtensionsManagerPrivate));
-
-	languages = g_get_language_names ();
-	g_assert (languages != NULL);
 }
